'use strict';

var fs = require('fs');
var path = require('path');
var scssBundle = require('scss-bundle');

function bundleScss({ output, exclusive = true, bundlerOptions = {} } = {}) {
  const files = [];
  const {
    project = null,
    dedupeGlobs = [],
    includePaths = [],
    ignoreImports = [],
  } = bundlerOptions;
  return {
    name: 'bundle-scss',
    transform(source, id) {
      if (/\.s?css$/.test(id)) {
        files.push({ id, content: source });
        if (exclusive) {
          return { code: `export default ${JSON.stringify(source)}` };
        }
      }
      return null;
    },
    async generateBundle(opts) {
      const outputPath = path.resolve(
        opts.file ? path.dirname(opts.file) : opts.dir,
        output || `${opts.file ? path.parse(opts.file).name : 'index'}.scss`,
      );
      await fs.promises.mkdir(path.dirname(outputPath), { recursive: true });
      const entryContent = files.map((file) => `@import "${file.id}";`).join('\n');
      await fs.promises.writeFile(outputPath, entryContent);
      const registry = Object.assign({}, ...files.map((file) => ({ [file.id]: file.content })));
      const bundler = new scssBundle.Bundler(registry, project);
      const result = await bundler.bundle(outputPath, dedupeGlobs, includePaths, ignoreImports);
      await fs.promises.writeFile(outputPath, result.bundledContent);
    },
  };
}

module.exports = bundleScss;
